@@ -11,6 +11,7 @@ DEFAULT_OPTIONS = {
         'daemon': False,
         'instance': 'default',
         'dry_run': False,
+        'pid_file_dir': '/tmp',
 
         # [global]
         'region': 'us-east-1',
@@ -27,6 +28,8 @@ DEFAULT_OPTIONS = {
         'log_config_file': None
     },
     'table': {
+        'enable_reads_autoscaling': True,
+        'enable_writes_autoscaling': True,
         'reads_lower_threshold': 30,
         'reads_upper_threshold': 90,
         'throttled_reads_upper_threshold': 0,
@@ -53,6 +56,8 @@ DEFAULT_OPTIONS = {
         'sns_message_types': []
     },
     'gsi': {
+        'enable_reads_autoscaling': True,
+        'enable_writes_autoscaling': True,
         'reads_lower_threshold': 30,
         'reads_upper_threshold': 90,
         'throttled_reads_upper_threshold': 0,
@@ -186,6 +191,12 @@ def __get_config_table_options(conf_file_options):
                 for option in DEFAULT_OPTIONS['gsi'].keys():
                     opt = DEFAULT_OPTIONS['gsi'][option]
 
+                    if 'gsis' not in options[table_name]:
+                        options[table_name]['gsis'] = {}
+
+                    if gsi_name not in options[table_name]['gsis']:
+                        options[table_name]['gsis'][gsi_name] = {}
+
                     if (option not in conf_file_options[
                             'tables'][table_name]['gsis'][gsi_name]):
                         options[table_name]['gsis'][gsi_name][option] = opt
@@ -207,12 +218,6 @@ def __get_config_table_options(conf_file_options):
                         opt = conf_file_options[
                             'tables'][table_name]['gsis'][gsi_name][option]
 
-                    if 'gsis' not in options[table_name]:
-                        options[table_name]['gsis'] = {}
-
-                    if gsi_name not in options[table_name]['gsis']:
-                        options[table_name]['gsis'][gsi_name] = {}
-
                     options[table_name]['gsis'][gsi_name][option] = opt
 
     return options
