@@ -58,6 +58,10 @@ def parse():
             'Used to run multiple instances of Dynamic DynamoDB. '
             'Give each instance a unique name and control them separately '
             'with the --daemon flag. (default: default)'))
+    daemon_ag.add_argument(
+        '--pid-file-dir',
+        default='/tmp',
+        help='Directory where pid file is located in. Defaults to /tmp')
     dynamodb_ag = parser.add_argument_group('DynamoDB options')
     dynamodb_ag.add_argument(
         '-r', '--region',
@@ -72,6 +76,12 @@ def parse():
         help="""Scale up the reads with --increase-reads-with percent if
                 the currently consumed read units reaches this many
                 percent (default: 90)""")
+    r_scaling_ag.add_argument(
+        '--throttled-reads-upper-threshold',
+        type=int,
+        help="""Scale up the reads with --increase-reads-with percent if
+                the count of throttled read events exceeds this
+                count (default: 100)""")
     r_scaling_ag.add_argument(
         '--reads-lower-threshold',
         type=int,
@@ -111,6 +121,12 @@ def parse():
         help="""Scale up the writes with --increase-writes-with percent
                 if the currently consumed write units reaches this
                 many percent (default: 90)""")
+    w_scaling_ag.add_argument(
+        '--throttled-writes-upper-threshold',
+        type=int,
+        help="""Scale up the reads with --increase-writes-with percent if
+                the count of throttled write events exceeds this
+                count (default: 100)""")
     w_scaling_ag.add_argument(
         '--writes-lower-threshold',
         type=int,
